﻿Imports System.Activities
Imports System.Activities.Statements
Imports System.Activities.DurableInstancing
Imports System.Runtime.DurableInstancing
Imports System.Threading
Imports Microsoft.Samples

'-----------------------------------------
' note: running this code requires setting up
'       a WF4.5 instance store
'-----------------------------------------
Module Module1

    Private useIdentity As Boolean = False

    Sub Main()
        ' start an instance, unload it, and persist it 
        ' into the instance store
        Dim instanceId As Guid = StartAndUnloadInstance

        ' load the instance with the wrong definition 
        LoadAndCompleteInstanceWithWrongDefinition(instanceId)

        Console.WriteLine("Press [Enter] to exit.")
        Console.ReadLine()

    End Sub

    Private Function StartAndUnloadInstance() As Guid
        ShowMessageFromHost("Starting instance", ConsoleColor.Green)

        ' ****************************************
        ' Create the workflow application
        ' ****************************************
        Dim application As WorkflowApplication = Nothing
        If useIdentity Then
            ' with identity             
            Dim identityV1 As New WorkflowIdentity("WorkflowIdentitySample", New Version(1, 0, 0, 0), Nothing)
            application = New WorkflowApplication(New Activity1, identityV1)
            ShowMessageFromHost(String.Format("Creating with identity '{0}'", application.DefinitionIdentity.ToString))
        Else
            application = New WorkflowApplication(New Activity1)
            ShowMessageFromHost("Creating without identity")
        End If

        ' configure the workflow application and start executing            
        Dim syncEvent As New AutoResetEvent(False)
        application.InstanceStore = SetupInstanceStore
        application.PersistableIdle = Function(e) PersistableIdleAction.Unload
        application.Unloaded = Sub(e)
                                   ShowMessageFromHost("Persisted and unloaded from memory")
                                   syncEvent.Set()
                               End Sub
        application.Run()

        syncEvent.WaitOne()

        Return application.Id
    End Function

    Private Sub LoadAndCompleteInstanceWithWrongDefinition(ByVal instanceId As Guid)
        Dim input As String = Console.ReadLine

        Try
            ShowMessageFromHost("Reloading instance")

            '   We are attempting to load the current instance
            '   using a different version of the program definition!!!        
            Dim application As WorkflowApplication = Nothing
            If useIdentity Then
                ' with identity:
                Dim identityV2 As New WorkflowIdentity("WorkflowIdentitySample", New Version(2, 0, 0, 0), Nothing)
                application = New WorkflowApplication(New Activity2, identityV2)
                ShowMessageFromHost(String.Format("Loading with identity '{0}'", application.DefinitionIdentity.ToString))
            Else
                ' without identity:
                application = New WorkflowApplication(New Activity2)
            End If

            ' configure the workflow application
            Dim syncEvent As New AutoResetEvent(False)
            application.InstanceStore = SetupInstanceStore
            application.Unloaded = Function(e) syncEvent.Set

            ' load the application (will fail due to version mismatch)                
            application.Load(instanceId)

            ' this resumes the bookmark setup by readline           
            application.ResumeBookmark("ReadLine", input)
            syncEvent.WaitOne()
        Catch ex As VersionMismatchException
            ShowMessageFromHost(String.Format("A {0} occurred.", ex.GetType.Name), ConsoleColor.Red)
            ShowMessageFromHost(String.Format("Message: {0}", ex.Message), ConsoleColor.Red)
        Catch ex As Exception
            ShowMessageFromHost(String.Format("A {0} occurred.", ex.GetType.Name), ConsoleColor.Red)
            ShowMessageFromHost(String.Format("   Message: {0}", ex.Message), ConsoleColor.Red)
        End Try
    End Sub

    Private Function SetupInstanceStore() As InstanceStore
        Dim instanceStore As InstanceStore = New SqlWorkflowInstanceStore("Data Source=.\SQLExpress;Initial Catalog=InstanceStore;Integrated Security=True")

        Dim handle As InstanceHandle = instanceStore.CreateInstanceHandle
        Dim view As InstanceView = instanceStore.Execute(handle, New CreateWorkflowOwnerCommand, TimeSpan.FromSeconds(30))
        handle.Free()

        instanceStore.DefaultInstanceOwner = view.InstanceOwner

        Return instanceStore
    End Function

    Private Sub ShowMessageFromHost(ByVal message As String, Optional ByVal color As ConsoleColor = ConsoleColor.Yellow)
        Dim currentColor = Console.ForegroundColor
        Console.ForegroundColor = color
        Console.WriteLine("[Host] {0}", message)
        Console.ForegroundColor = currentColor
    End Sub

End Module
